import 'package:flutter/material.dart';

enum AnimationType {
  normal,
  fadeIn,
  rotation,
  sacle,
  slide,
}

class RouteTransition {
  RouteTransition(
    this.context, {
    @required this.child,
    this.replacement = false,
    this.duration = const Duration(milliseconds: 250),
    this.type = AnimationType.normal,
  }) {
    switch (type) {
      case AnimationType.normal:
        _normalTransition();
        break;
      case AnimationType.fadeIn:
        _fadeInTransition();
        break;
      case AnimationType.rotation:
        _rotationTransition();
        break;
      case AnimationType.sacle:
        _scaleTransition();
        break;
      case AnimationType.slide:
        _slideTransition();
        break;
    }
  }

  final Widget child;
  final bool replacement;
  final Duration duration;
  final AnimationType type;
  final BuildContext context;

  void _normalTransition() =>
      _navigate(MaterialPageRoute(builder: (context) => child));

  void _fadeInTransition() {
    final route = PageRouteBuilder(
      pageBuilder: (_, __, ___) => child,
      transitionDuration: duration,
      transitionsBuilder: (_, animation, __, child) => FadeTransition(
        child: child,
        opacity: Tween<double>(begin: 0.0, end: 1.0).animate(
          CurvedAnimation(
            parent: animation,
            curve: Curves.easeOut,
          ),
        ),
      ),
    );
    _navigate(route);
  }

  void _rotationTransition() {
    final route = PageRouteBuilder(
      pageBuilder: (_, __, ___) => child,
      transitionDuration: duration,
      transitionsBuilder: (_, animation, __, child) => RotationTransition(
        child: child,
        turns: Tween<double>(begin: 0.0, end: 1.0).animate(
          CurvedAnimation(
            parent: animation,
            curve: Curves.easeOut,
          ),
        ),
      ),
    );
    _navigate(route);
  }

  void _scaleTransition() {
    final route = PageRouteBuilder(
      pageBuilder: (_, __, ___) => child,
      transitionDuration: duration,
      transitionsBuilder: (_, animation, __, child) => ScaleTransition(
        child: child,
        scale: Tween<double>(begin: 0.0, end: 1.0).animate(
          CurvedAnimation(
            parent: animation,
            curve: Curves.easeOut,
          ),
        ),
      ),
    );
    _navigate(route);
  }

  void _slideTransition() {
    final route = PageRouteBuilder(
      pageBuilder: (_, __, ___) => child,
      transitionDuration: duration,
      transitionsBuilder: (_, animation, __, child) => SlideTransition(
        child: child,
        position:
            Tween<Offset>(begin: Offset(0.5, 1.0), end: Offset.zero).animate(
          CurvedAnimation(
            parent: animation,
            curve: Curves.easeOut,
          ),
        ),
      ),
    );
    _navigate(route);
  }

  void _navigate(Route route) {
    (replacement)
        ? Navigator.pushReplacement(context, route)
        : Navigator.push(context, route);
  }
}

// https://flutter.dev/docs/development/packages-and-plugins/developing-packages
